
car
	parent_type = /mob

	icon = 'car.png'
	animate_movement = NO_STEPS

	var
		// Car interaction variables.
		accelerating = 0  // A flag that determines whether the accelerate key is held
		turning = 0 // -1 for left, 1 for right
		braking = 0 // if 1, then the car slams on its brakes!
		angle = 20 // car's rough angle, anywhre from 0 to 359
		velocity = 0 // car's current speed
		dx = 0  // The x position within an individual tile
		dy = 0  // The y position within an individual tile

		// Car specification variables.
		states = 160
		turn_rate = 0.8
		max_velocity = 18
		max_reverse = -5
		acceleration = 0.1
		deceleration = 0.1
		brake_speed = 0.2


	// When a new car is created.
	New(loc)

		// Add this car to the list of cars in the world.
		CarsInWorld += src

		// When the car is newly created, make sure its facing the proper direction.
		src.SetAngleImage()

		return ..(loc)


	proc

		// This sets the car's icon state to match the angle it is facing.
		SetAngleImage()
			var/state = (src.states - round(src.angle / 360 * src.states))-1
			src.icon_state = "0,[state]"
			return


		// This is called when the car is told to turn, either left or right.
		// For 'direction', 1 is right, -1 is left.
		Turn(direction = 0)

			// Update the angle.
			switch(direction)
				if(1) // right
					src.angle += src.turn_rate
				if(-1) // left
					src.angle -= src.turn_rate

			// If the angle is out of range wrap the angle back around to the right range
			while(src.angle >= 360)
				src.angle -= 360
			while(src.angle < 0)
				src.angle += 360

			src.SetAngleImage()
			return


		// This is what moves the car.
		Drive()
			var/turf/T
			var/cx
			var/cy

			// Break down the components of the velocity into x and y chunks,
			// then add them to the x and y position within the tile.
			src.dx += sin(src.angle) * src.velocity
			src.dy += cos(src.angle) * src.velocity

			// Check to see if the player has moved off the tile and by how many tiles.
			while(src.dx > tile_radius)
				src.dx -= tile_radius * 2
				cx++
			while(src.dx < -tile_radius)
				src.dx += tile_radius * 2
				cx--
			while(src.dy > tile_radius)
				src.dy -= tile_radius * 2
				cy++
			while(src.dy < -tile_radius)
				src.dy += tile_radius * 2
				cy--

			// Move the player tile by tile through all the tiles that they passed through
			// to make sure that there aren't any dense tiles in the way.
			while(abs(cx) > 0 || abs(cy) > 0)
				var/i
				var/j

				if(cx > 0)
					i++
					cx--
				if(cx < 0)
					i--
					cx++
				if(cy > 0)
					j++
					cy--
				if(cy < 0)
					j--
					cy++

				T = locate(x+i, y+j, z)

				if(T)
					src.Move(T)

			// Set the pixel offsets for the tile tey are on.
			pixel_x = (src.dx * 16/tile_radius)
			pixel_y = (src.dy * 16/tile_radius)

			// If the car has a client, set the client's pixel offsets as well.
			if(src.client)
				src.client.pixel_x = src.pixel_x
				src.client.pixel_y = src.pixel_y



